/******************************************************************************
 * test_analysis_x.cpp                                                        *
 *                                                                            *
 * Top Level file in analysis package                                         *
 * Drives the package                                                         *
 *****************************************************************************/
#define Global_cxx
#include "single_top_analysis/AnalysisMain.hpp"
#include <iostream>
#include <string>

// Include histogramming classes
#include "single_top_analysis/EventWeight.hpp"
#include "single_top_analysis/CutNJets.hpp"
#include "single_top_analysis/CutElectron.hpp"
#include "single_top_analysis/HistogrammingNJets.hpp"
#include "single_top_analysis/HistogrammingElectron.hpp"
#include "single_top_analysis/HistogrammingMuon.hpp"
#include "single_top_analysis/HistogrammingTau.hpp"
#include "single_top_analysis/HistogrammingMET.hpp"
#include "single_top_analysis/HistogrammingJet.hpp"
#include "single_top_analysis/HistogrammingMCParticle.hpp"
#include "single_top_analysis/HistogrammingTopology.hpp"

// Include cuts classes
#include "single_top_analysis/CutBasic.hpp"

using std::cout;
using std::endl;

/******************************************************************************
 * int main(int argc, char **argv)                                            *
 *                                                                            *
 * Main Routine                                                               *
 *                                                                            *
 * Input:  Command line arguements                                            *
 * Output: Returns 0 of successful                                            *
 ******************************************************************************/
// Only declare main if not in CINT (not interactive?)
#ifndef __CINT__
int main(int argc, char **argv) 
#endif
{
  cout << "<driver> Starting parton_analysis" << endl;

  /////////////////////////////////////////////////////////////////////////////////
  // Decleartions and Instantiations
  /////////////////////////////////////////////////////////////////////////////////
  // Instantiate the analysis class
  AnalysisMain mystudy;

  // Delcare and Instantiate chains for the rootuples
  TChain *chainCollection = new TChain("CollectionTree");
  TChain *chainEVO        = new TChain("EV0");
  TChain *chainTruth      = new TChain("Truth0");
  
  /////////////////////////////////////////////////////////////////////////////////
  // Parse the Command Line
  /////////////////////////////////////////////////////////////////////////////////
  Int_t parseReturnValue = mystudy.ParseCmdLine(argc,argv,chainCollection,chainEVO,chainTruth);
  if(parseReturnValue != 0) return parseReturnValue;
 
  // Check command line for Fast Running
  // In Fast Running only a few cuts are made and histograms filled
  Bool_t doFast = kFALSE;
  for (int i = 1; i < argc; ++i) {
    //cout<<"Command line parameter "<<i<<" is "<<argv[i]<<endl;
    if (!strcmp(argv[i], "-fast")) {
      doFast = kTRUE;
      cout << "Driver: Fast running " << endl;
    }//if fast
  } // for
  

  /////////////////////////////////////////////////////////////////////////////////
  // Define a Particle class to be the same as the AnalysisMain class
  /////////////////////////////////////////////////////////////////////////////////
  EventContainer *particlesObj = &mystudy;
  
  /////////////////////////////////////////////////////////////////////////////////
  // Add Cuts and Histograms applicable to Fast and Full Analyses
  /////////////////////////////////////////////////////////////////////////////////
  // First always add the EventWeight
  mystudy.AddCut(new EventWeight(particlesObj));
  // Histograms: Number of Jets
  mystudy.AddCut(new HistogrammingNJets(particlesObj));
  // Histograms: Some MC particle properties
  mystudy.AddCut(new HistogrammingMCParticle(particlesObj));
  // Histograms: Electron Properties
  mystudy.AddCut(new HistogrammingElectron(particlesObj));
  // Histograms: Muon Properties
  mystudy.AddCut(new HistogrammingMuon(particlesObj));
  // Histograms: Tau Properties
  mystudy.AddCut(new HistogrammingTau(particlesObj));
  // Histograms: MET
  mystudy.AddCut(new HistogrammingMET(particlesObj));
  // Histograms: Jet Properties
  mystudy.AddCut(new HistogrammingJet(particlesObj));
  // Cut: Number of electrons
  mystudy.AddCut(new CutElectron(particlesObj));
  // Cut: Number of Jets
  mystudy.AddCut(new CutNJets(particlesObj));
  // Histograms: Number of Jets
  mystudy.AddCut(new HistogrammingNJets(particlesObj));

  // Finally, make histograms of topological variables and write out
  // a separate skim tree with them
  mystudy.AddCut(new HistogrammingTopology(particlesObj));
  

  
  cout << "<driver> Starting Loop over events in chain" << endl << endl;
    
  // Loop over events in chain
  mystudy.Loop();
  cout << "<driver> Finished Loop over events in chain" << endl;

  // Write output to files
  cout << "<driver> " << "Writing histograms to file: " << mystudy.GetHistogramFileName() << endl;
  
  mystudy.Finish();
  
  // Delete chains and set pointers to NULL
  cout << "<driver> Deleting chains" << endl;

  if(chainTruth)      delete chainTruth;
  if(chainEVO)        delete chainEVO;
  if(chainCollection) delete chainCollection;

  chainTruth      = NULL;
  chainEVO        = NULL;
  chainCollection = NULL;
  
  cout << "<driver> Sucessful Completion." << endl;
  return 0;
}
