/////////////////////////////////////////////////////////////////////////
//
// This macro plots a bunch of histograms one on top of each other with
// different color lines and a legend (_WITHOUT_ stat boxes) for each one of
// them. The user specifies the order of colrs through a palette, like: 
// Int_t palette[9]={1,2,3,4,5,6,7,8,9};
// 
// It takes as input a TObjArray of TH1F, the main title, the x and y axis
// labels and the palette. It will calculate the dimensions of the pad
// automatically based on the largest histograms. 
//
// original implementation on 10/15/2003 by Aran Garcia-Bellido (aran@fnal.gov)
//
//
// Modified 3/2/04 AGB. Made legend transparent, with bigger font and somewhat
// more centered. Make sure we don't get any stabox at the back. 
//
// Modified 5/2/2004, Reinhard Schwienhorst: Change to display curves separately
//                                           and in different line styles.
//
/////////////////////////////////////////////////////////////////////////


void CombineHistos(TObjArray histograms,
		   TString *legends, 
		   TString title="Main Title",
		   TString xtitle="X title",
		   TString ytitle="y title",
		   Int_t *palette,
		   Int_t *style,
		   Int_t legendCorner=1,  // display legend, and if yes in which corner of the plot?
		                          // =-1: no legend, =1: automatic determination
		                          // legend corners: 10 upper left
		                          //                 20 upper right
		                          //                 11 lower left
		                          //                 21 lower right
		                          //                 31 centered 
		                          //                 32 middle top
		   Int_t smoothen=300,    // smoothen the histogram?
		   Int_t rebin=1,         // rebin the histogram by this much
		   Int_t drawLine=1,    // 0:histogram, 1:line, 2:both
		   bool normalize=false,  // normalize histograms to unit area?
		   float xmin_in=-99999.,     // min or max for x-axis?
		   float xmax_in=-99999.,
		   float xmin_cutoff=-99999.,  // min cutoff after smoothing for X axis or to zero bins?
		   float ymin_in=-99999.,  // min or max for y-axis?
		   float ymax_in=-1.,      // 
		   float legendXMin=-1.    // x min corner for legend box
		   ){
  // Number of histos to plot:
  Int_t ntot = histograms.GetEntries();
  TH1F *h;
  
  // First, check largest dimensions so that we can plot all histograms at once:
  Float_t xmin, xmax, ymin=99., ymax=-99.;

  bool withLegend=(legendCorner>=0);
  // change the location of the legend depending on where the plot is
  double lgdx1=0.75;
  double lgdy1=0.91-0.06*ntot;
  double lgdy2=0.91;

  // define a few additional line styles:
  gStyle->SetLineStyleString(5,"20 12 4 12");
  gStyle->SetLineStyleString(6,"20 12 4 12 4 12 4 12");
  gStyle->SetLineStyleString(7,"20 20");
  gStyle->SetLineStyleString(8,"20 12 4 12 4 12");
  gStyle->SetLineStyleString(9,"80 20");
  gStyle->SetLineStyleString(10,"50 10 10 10");
  gStyle->SetLineStyleString(11,"25 8");
  gStyle->SetLineStyleString(20,"60 20");
  
  // now loop over the histograms and set them up
  for(Int_t i = 0; i<ntot; i++){
    h = (TH1F*) histograms[i];
    // should we normalize first?
    // if yes we have to clone the histogram so that we don't
    // modify the original.
    if(normalize) {
      TH1F *thist=h->Clone("temp");
      h=thist;
      histograms[i]=h;
      float scaleFactor=((TH1F*)histograms[0])->Integral()/h->Integral();
      h->Scale(scaleFactor);
    }
    //
    // should we rebin? In that case we also have to clone.
    if(rebin>1) {
      TH1F *thist=(TH1F*)h->Clone("temp");
      h=thist;
      histograms[i]=h;
      h->Rebin(rebin);
    }

    //
    // should we set bin contents below xmin_cutoff to 0?
    if(drawLine==0 && xmin_cutoff>0.) {
      for(int iBin=0;iBin<h->GetNbinsX();iBin++) {
	if(h->GetBinCenter(iBin)<xmin_cutoff) h->SetBinContent(iBin,0.);
	else break;
      } // end of loop over bins
    } // end of: if resetting bins below xmin_cutoff

    TAxis *axis = h->GetXaxis();
    if( axis->GetXmin() < xmin ) xmin = axis->GetXmin();
    if( axis->GetXmax() > xmax ) xmax = axis->GetXmax();
    if( h->GetMinimum() < ymin ) ymin = h->GetMinimum();
    if( h->GetMaximum() > ymax ) ymax = h->GetMaximum();
    // also check where bin 0 is on the vertical axis
    //cout<<"y(max)= "<<h->GetBinContent(1)<<endl;
    if(h->GetBinContent(h->GetNbinsX()+1)<(ymax-ymin)/2.+ymin) {
      lgdy1=0.91-0.06*ntot;
      if( h->GetMaximum() >= ymax ) {
	//cout<<"max bin is at "<<h->GetMaximumBin()<<", n "<<h->GetNbinsX()<<endl;
	if(h->GetMaximumBin()/(float)h->GetNbinsX()>0.58) lgdx1=0.25;
	else lgdx1=0.7;
      } // end of: if new max
    }
    else {
      lgdy1=0.17;
      if( h->GetMinimum() <= ymin ) {
	if(h->GetMinimumBin()/(float)h->GetNbinsX()>0.58) lgdx1=0.25;
	else lgdx1=0.7;
      } // end of: if new max
    }
  } // end of initial loop over histograms

  // allow for re-scaling of some of the histogram
  if(xmin_in>=-99998.) xmin=xmin_in;
  if(xmax_in>=0.) xmax=xmax_in;
  if(ymin_in>-99999.) ymin=ymin_in;
  if(ymax_in>=0.) ymax=ymax_in;

  ymax = ymax*1.15;
  ymin = ymin*1.05;
  TH1F * h0 = (TH1F*) histograms[0];
  Int_t nbinsx = h0->GetXaxis()->GetNbins();
  Int_t nbinsy = 100;
  //cout<<"ymin is "<<ymin<<", ymax is "<<ymax<<endl;

  // Now make the plots:
  TH2F * frame=0;
  // check if this histogram already exists in memory. If yes, don't bother creating a new one.
  frame = (TH2F*) gROOT->FindObject("frame");
  
  if(frame==NULL) {
    frame = new TH2F("frame","",nbinsx,xmin,xmax,nbinsy,ymin,ymax);
  }
  else {
    frame->SetBins(nbinsx,xmin,xmax,nbinsy,ymin,ymax);
  }

  // set the title in case we want to display it.
  frame->SetTitle("");

  frame->SetXTitle(xtitle.Data()); 
  frame->SetYTitle(ytitle.Data()); 
  frame->SetStats(false);

  // for some histograms we may need a large title offset so that numbers don't overlap.
  // This could be up to 1.7.
  frame->GetYaxis()->SetTitleOffset(1.1);
  // try to fix axis division if it's screwed up
  frame->GetXaxis()->SetNdivisions(505,true);
  // make the axis title in black
  frame->GetXaxis()->SetTitleColor(1);
  frame->GetYaxis()->SetTitleColor(1);

  frame->Draw("9");

  // TLegend(x1, y1, x2, y2)
  // force the legend to a specific corner if so desired by the user.
  // legend corners: 10 upper left
  //                 20 upper right
  //                 11 lower left
  //                 21 lower right
  //                 31 centered in X and Y
  //                 32 centered in X, upper edge in Y
  if(legendCorner==10) {
    lgdx1=0.25;
    lgdy1=0.91-0.06*ntot;
  }
  else if(legendCorner==20) {
    lgdx1=0.75;
    lgdy1=0.91-0.06*ntot;
  }
  else if(legendCorner==11) {
    lgdx1=0.25;
    lgdy1=0.17;
  }
  else if(legendCorner==21) {
    lgdx1=0.75;
    lgdy1=0.17;
  }
  else if(legendCorner==31) {
    lgdx1=0.5;
    lgdy1=0.4;
  }
  else if(legendCorner==32) {
    lgdx1=0.6;
    lgdy1=0.91-0.06*ntot;
  }
  // should we force the legend into a specific corner?
  if(legendXMin>-1.) lgdx1=legendXMin;
  
  // the y1 box for the legend should depend on the number of entries:
  double lgddx=0.14;
  TLegend *lgd = new TLegend(lgdx1,lgdy1,lgdx1+lgddx,lgdy1 + 0.06*ntot); // AGB used to be (.7,.7,.89,.89)
  // This line makes the legend transparent (not grey, ewwww!): 
  lgd->SetBorderSize(0); 
  lgd->SetTextSize(0.055); 
  lgd->SetTextFont(22); 
  lgd->SetFillColor(0);


  // now draw the rest
  for(Int_t iCh = 0; iCh<ntot; iCh++){

    //cout<<"looping through histogram "<<i<<", ptr is "<<histograms[i]<<endl;
    h = (TH1F*) ((TH1F*) histograms[iCh])->Clone("temp1");

    // set up colors and styles for the histograms
    //h->SetMarkerSize(1.5);
    h->SetMarkerStyle(0);
    h->SetLineColor(palette[iCh]);
    //h->SetLineSize(2.0);
    h->SetLineStyle(style[iCh]);
    h->SetStats(kFALSE);
    h->SetStats(0);

    // 
    // smoothen the histogram so that we can draw a nice line
    // if so desired.
    if(smoothen>-1 && drawLine>0) {
      // plot before smoothing to show that it's ok
      if(drawline==2) {
	TH1F *h1=(TH1F*)(h->Clone());
	h1->Draw("same");
      }
      // before smoothing, check if there are histogram entries
      // with zero entries on the left
      int iShiftMin=0;
      int iShiftMax=h->GetNbinsX();
      //cout<<"Checking shift"<<endl;
      for(int is=1;is<h->GetNbinsX();is++) {
	if(xmin_cutoff>-99999.) {
	  //user-defined lower edge
	  if(h->GetBinCenter(is)<xmin_cutoff) iShiftMin=is;
	}
	else {
	  // not user-defined, find out myself:
	  if(h->GetBinContent(is)!=0.) {
	    iShiftMin=is;  
	    break;
	  }
	}
      }
      // user-defined lower limit?

      // also find the upper limit
      for(int is=h->GetNbinsX();is>1;is--) {
	if(h->GetBinContent(is)!=0.) {
	  iShiftMax=is+1;  
	  break;
	}
      }
      //cout<<"ishift min is "<<iShiftMin<<" and max is "<<iShiftMax<<endl;
      // now use code copied here from the Smooth() method
      // to do the smoothing only for a limited range:
      Double_t *XX = new Double_t[iShiftMax-iShiftMin];
      for (int is=iShiftMin;is<iShiftMax;is++) {
	XX[is-iShiftMin] = h->GetBinContent(is);
      }
      TH1::SmoothArray(iShiftMax-iShiftMin,XX,smoothen);
      //
      // and copy the result back to the histogram
      for (int is=iShiftMin;is<iShiftMax;is++) {
	h->SetBinContent(is,XX[is-iShiftMin]);
      }
      // and clean up the intermediate array
      delete [] XX;
     
    } // end of: if smoothen
    if(drawLine>0) {
      // and draw in the existing figure, and change to draw a line!
      h->Draw("SAME L");
    }
    else {
      h->Draw("SAME");
    }
    
    lgd->AddEntry(h,legends[iCh].Data(),"l");

    // now that we are done drawing, scale the histogram back to its original size


  } // end of loop over histograms

  //cout<<"Done! now drawing legend"<<endl;

  if(withLegend) lgd->Draw("SAME");

  //cout<<"Done with CombineHistos."<<endl;
} // end of: CombineHistos.C




Float_t FindMax(TH1F* _histo1,TH1F* _histo2 )
{
  // Find max and min
  Float_t data_max = _histo1->GetMaximum();
  Float_t back_max = _histo2->GetMaximum(); 
  Float_t max = data_max;
  if ( back_max > max )
    max = back_max;

  return( max );
}





