/******************************************************************************
 * Electron.hpp                                                               *
 *                                                                            *
 * Store information about final state electrons                              *
 *                                                                            *
 * Derived from Particle class                                                *
 *                                                                            *
 * Public Member Functions of Electron class                                  *
 *    Electron()                         -- Default Constructor               *
 *    Electron()                         -- Parameterized Constructor         *
 *    Clear()                            -- Set contents to default           *
 *    Fill                               -- Fill electron from event tree     *
 *    SetEoverP                          -- Set E over P for electron         *
 *    GetEoverP                          -- GetEoverP                         *
 *    operator+=                         -- Overload +=                       *
 *    operator+                          -- Overload +                        *
 *    operator=                          -- Overload = Particle               *
 *    operator=                          -- Overload = const Electron         *
 *    operator =                         -- Overload =                        *
 *                                                                            *
 * Private Data Members of Electron class                                     *
 *    Double_t _EoverP;                  -- E over P                          *
 *                                                                            *
 * History                                                                    *
 *      19 Oct 2006 - Created by R. Schwienhorst                              *
 *      17 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#ifndef electron_h
#define electron_h

#include "single_top_analysis/Particle.hpp"
#include "single_top_analysis/EventTree.hpp"

class Electron: public Particle
{
 public:

  // Default constructor
  Electron();

  // Copy constructor for Electrons
  Electron(const Electron& other);

  // Copy constructor for Particles
  Electron(const Particle& other);

  // Destructor
  ~Electron();

  // Set all contents to their defaults
  inline void Clear() { Particle::Clear(); _EoverP = 0.0; }

  // Fill the electron from an EventTree
  void Fill(EventTree *evtr, Int_t iE);

  // Get and Set E over P for electron
  inline void SetEoverP(const Double_t& eop) { _EoverP = eop; }
  inline Double_t GetEoverP() const { return _EoverP; }

  // Overloaded operators:
  // +=
  Electron& operator+=(const Electron& other);
  // +
  Electron operator+(const Electron& other);
  // = Particle
  Electron& operator=(const Particle& other);
  // = Electron const
  Electron& operator=(const Electron& other);
  // = Electron non-const
  Electron& operator=(Electron& other);

 private:

  // CAL energy of the electron divided by track momentum
  Double_t _EoverP;  

  ////////////////////////////////////////////////////////////////////////////////
  // Integrate classes into the Root system
  // Must come at end of class definition
  ClassDef(Electron,0)
  ////////////////////////////////////////////////////////////////////////////////
};


#endif
