/******************************************************************************
 * Jet.hpp                                                                    *
 *                                                                            *
 * Store information about final state jets                                   *
 *                                                                            *
 * Derived from Particle class                                                *
 *                                                                            *
 * Public Member Functions of Jet class                                       *
 *    Jet()                              -- Default Constructor               *
 *    Jet()                              -- Parameterized Constructor         *
 *    ~Jet()                             -- Destructor                        *
 *    Clear()                            -- Set contents to default           *
 *    Fill                               -- Fill electron from event tree     *
 *    SetEtM0                            -- Set ET for EM layer 0             *
 *    GetEtM0                            -- Set ET for EM layer 0             *
 *    EtEM0                              -- Set ET for EM layer 0             *
 *    operator+=                         -- Overload +=                       *
 *    operator+                          -- Overload +                        *
 *    operator=                          -- Overload = Particle               *
 *    operator=                          -- Overload = const Jet              *
 *    operator=                          -- Overload =                        *
 *                                                                            *
 * Private Data Members of Jet class                                          *
 *    Double_t _EtEm0                     -- Et for ME layer 0                *
 *                                                                            *
 * History                                                                    *
 *      16 Nov 2006 - Created by R. Schwienhorst                              *
 *      17 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#ifndef jet_h
#define jet_h

#include "single_top_analysis/Particle.hpp"
#include "single_top_analysis/EventTree.hpp"
//

class Jet: public Particle
{
 public:
  // Default Constructor
  Jet();

  // Copy Constructor
  Jet(const Jet& other);

  // Construtor using particle
  Jet(const Particle& other);

  // Destructor
  ~Jet();

  // Set all contents to their defaults
  inline void Clear() { Particle::Clear(); _EtEM0=0.; };

  // Fill the jet from an EventTree
  void Fill(EventTree *evtr,Int_t iE);

  // Get and Set Et for EM level 0
  inline void SetEtEM0(const Double_t& eop) { _EtEM0=eop; };
  inline Double_t GetEtEM0() const { return _EtEM0; };
  inline Double_t EtEM0() const { return GetEtEM0(); };

  // Overloaded Operators
  // +=
  Jet& operator+=(const Jet& other);
  // +
  Jet operator+(const Jet& other);
  // = Jet const
  Jet& operator=(const Particle& other);
  // = Particle const
  Jet& operator=(const Jet& other);
  // = Jet non-const
  Jet& operator=(Jet& other);
    

 private:
  // CAL energy in first EM layer
  Double_t _EtEM0;   

  ////////////////////////////////////////////////////////////////////////////////
  // Integrate classes into the Root system
  // Must come at end of class definition
  ClassDef(Jet,0)
  ////////////////////////////////////////////////////////////////////////////////
};


#endif
