/******************************************************************************
 * MCElectron.hpp                                                             *
 *                                                                            *
 * Container for a MCElectron                                                 *
 *                                                                            *
 * Derived from Particle class                                                *
 *                                                                            *
 * Public Member Functions of MCElectron class                                *
 *    MCElectron()                       -- Default Constructor               *
 *    MCElectron()                       -- Parameterized Constructor         *
 *    MCElectron()                       -- Constructor with Particle         *
 *    ~MCElectron()                      -- Destructor                        *
 *    Clear()                            -- Set contents to default           *
 *    Fill                               -- Fill electron from event tree     *
 *    SetEoverP                          -- Set E over P                      *
 *    GetEoverP                          -- Get E over P                      *
 *    EoverP                             -- Get E over P                      *
 *    operator+=                         -- Overload +=                       *
 *    operator+                          -- Overload +                        *
 *    operator=                          -- Overload = Particle               *
 *    operator=                          -- Overload = const MCElectron       *
 *    operator=                          -- Overload =                        *
 *                                                                            *
 * Private Data Members of MCElectron class                                   *
 *    Double_t _EoverP                   -- CAL Energy in cone around muon    *
 *                                                                            *
 * History                                                                    *
 *      15 Nov 2006 - Created by R. Schwienhorst                              *
 *      20 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/
#ifndef MCelectron_h
#define MCelectron_h

#include "single_top_analysis/MCParticle.hpp"
#include "single_top_analysis/TruthTree.hpp"

class MCElectron: public MCParticle
{
public:
  // Default Constructor
  MCElectron();
  // Parameterized Constructor
  MCElectron(const MCElectron& other);

  // Copy constructor from MCParticle
  MCElectron(const MCParticle& other);

  // Default destructor
  ~MCElectron();

  // Set all contents to their defaults
  inline void Clear() { MCParticle::Clear(); _EoverP=0.; };

  // Fill the MCelectron from a TruthTree
  void Fill(TruthTree *trtr, Int_t iE);

  // Setters
  // Set E over P for this MCelectron
  inline void SetEoverP(const Double_t& eop) { _EoverP = eop; };
  inline Double_t GetEoverP() const { return _EoverP; };
  inline Double_t EoverP() const { return GetEoverP(); };


  // Overloaded operators:
  // +=
  MCElectron& operator+=(const MCElectron& other);
  // +
  MCElectron operator+(const MCElectron& other);
  // = MCParticle const
  MCElectron& operator=(const MCParticle& other);
  // = MCElectron const
  MCElectron& operator=(const MCElectron& other);
  // = MCElectron non-const
  MCElectron& operator=(MCElectron& other);
    
private:
  
  // CAL energy of the MCelectron divided by track momentum
  Double_t _EoverP; 

  ////////////////////////////////////////////////////////////////////////////////
  // Integrate classes into the Root system
  // Must come at end of class definition
  ClassDef(MCElectron,0)
  ////////////////////////////////////////////////////////////////////////////////

};


#endif
