/******************************************************************************
 * MCParticle.hpp                                                             *
 *                                                                            *
 * Container for a MCParticle                                                 *
 *                                                                            *
 * Derived from Particle class                                                *
 *                                                                            *
 * Public Member Functions of MCParticle class                                *
 *    MCParticle()                       -- Default Constructor               *
 *    MCParticle()                       -- Parameterized Constructor         *
 *    MCParticle()                       -- Constructor with Particle         *
 *    ~MCParticle()                      -- Destructor                        *
 *    Clear()                            -- Set contents to default           *
 *    Fill                               -- Fill electron from event tree     *
 *    SetPdgId                           -- Set Particle Data Group ID        *
 *    SetStatus                          -- Set Status                        *
 *    SetBarCode                         -- Set Bar Code                      *
 *    GetPdgId                           -- Get Particle Data Group ID        *
 *    PdgId                              -- Get Particle Data Group ID        *
 *    GetAbsPdgId                        -- Get abs Particle Data Group ID    *
 *    AbsPdgId                           -- Get abs Particle Data Group ID    *
 *    GetStatus                          -- Get Status                        *
 *    Status                             -- Get Status                        *
 *    GetBarCode                         -- Get Bar Code                      *
 *    BarCode                            -- Get Bar Code                      *
 *    operator+=                         -- Overload +=                       *
 *    operator+                          -- Overload +                        *
 *    operator=                          -- Overload = Particle               *
 *    operator=                          -- Overload = const MCParticle       *
 *    operator=                          -- Overload =                        *
 *                                                                            *
 * Private Data Members of MCParticle class                                   *
 *    Int_t _PdgId                       -- PDG ID for this particle          *
 *    Int_t _Status                      -- Status flag for this particle     *
 *    Int_t _BarCode                     -- BarCode flag for this particle    *
 *                                                                            *
 * History                                                                    *
 *      15 Nov 2006 - Created by R. Schwienhorst                              *
 *      20 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#ifndef MCparticle_h
#define MCparticle_h

#include "single_top_analysis/Particle.hpp"
#include "single_top_analysis/TruthTree.hpp"
//

class MCParticle: public Particle
{
public:
  // Default constructor
  MCParticle();

  // Copy constructor
  MCParticle(const MCParticle& other);

  // Copy Constructor - Particle
  MCParticle(const Particle& other);

  // Destructor
  ~MCParticle();

  // Set all contents to their defaults
  inline void Clear() { Particle::Clear(); _PdgId = 0; _Status = 0; _BarCode = 0; };

  // Fill the MCparticle from a TruthTree
  void Fill(TruthTree *trtr, Int_t iE);

  // Setters PDG ID
  inline void SetPdgId(Int_t pdgId)   { _PdgId   = pdgId; };
  inline void SetStatus(Int_t pdgId)  { _Status  = pdgId; };
  inline void SetBarCode(Int_t pdgId) { _BarCode = pdgId; };

  // Getters PDG ID
  inline Int_t GetPdgId() const { return _PdgId; };
  inline Int_t PdgId() const { return GetPdgId(); };
  inline Int_t GetAbsPdgId() const { return(_PdgId>0 ? _PdgId : -_PdgId) ; };
  inline Int_t AbsPdgId() const { return GetAbsPdgId(); };

  // Status flag
  inline Int_t GetStatus() const { return _Status; };
  inline Int_t Status() const { return GetStatus(); };

  // Stable flag
  inline Int_t GetBarCode() const { return _BarCode; };
  inline Int_t BarCode() const { return GetBarCode(); };


  // Overloaded operators:
  // +=
  MCParticle& operator+=(const MCParticle& other);
  // +
  MCParticle operator+(const MCParticle& other);
  // = const Particle
  MCParticle& operator=(const Particle& other);
  // = MCParticle const
  MCParticle& operator=(const MCParticle& other);
  // = MC Particle const
  MCParticle& operator=(MCParticle& other);
    
private:
  Int_t _PdgId;    // PDG ID for this particle
  Int_t _Status;   // Status flag for this particle
  Int_t _BarCode;   // BarCode flag for this particle
  
  ////////////////////////////////////////////////////////////////////////////////
  // Integrate classes into the Root system
  // Must come at end of class definition
  ClassDef(MCParticle,0)
  ////////////////////////////////////////////////////////////////////////////////

};



#endif
