/******************************************************************************
 * CutCharge.cpp                                                              *
 *                                                                            *
 * Cuts on charge and flavor of lepton                                        *
 *                                                                            *
 * Derived from HistoCut which is in turn derived from BaseCut                *
 *                                                                            *
 * Public Member Functions of CutCharge class                                 *
 *    CutCharge()                       -- Parameterized Constructor          *
 *    ~CutCharge()                      -- Destructor                         *
 *    BookHistogram()                   -- Book histograms                    *
 *    Apply()                           -- Apply cuts and fill histograms     *
 *    GetCutName()                      -- Returns "CutCharge"                *
 *                                                                            *
 * Private Data Members of CutNJets                                           *
 *    Int_t _leptonCharge                -- lepton charge cut value           *
 *    Int_t _leptonFlavor                -- lepton flavor cut value           *
 *    myTH1F* _hLeptonCharge             -- Hist of lepton charge             *
 *    myTH1F* _hLeptonFlavor             -- Hist of lepton flavor             *
 *                                                                            *
 * History                                                                    *
 *      28 Apr 2004 - Created by R. Schwienhorst                              *
 *      14 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#include "single_top_analysis/CutCharge.hpp"
#include <iostream>

using namespace std;


/******************************************************************************
 * CutCharge::CutCharge(EventContainer *EventContainerObj)                    *     
 *                                                                            * 
 * Parameterized Constructor                                                  *
 *                                                                            *
 * Input:  Event Object class                                                 *
 * Output: None                                                               *
 *****************************************************************************/
CutCharge::CutCharge(EventContainer *obj) {
  SetEventContainer(obj);
} //CutCharge()


/******************************************************************************
 * CuCharge::~CutCharge()                                                     *
 *                                                                            *
 * Destructor                                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
CutCharge::~CutCharge()
{
}

/******************************************************************************
 * void CutCharge::BookHistogram()                                             *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void CutCharge::BookHistogram()
{

  // Histogram of events before cut on charge and flavor
  _hLeptonChargeBefore = DeclareTH1F("LeptonChargeBefore", "Sign of top quark charge before cut", 8, -6, 2);
  _hLeptonFlavorBefore = DeclareTH1F("LeptonFlavorBefore", "Flavor code (kf) of lepton before cut" ,50, -25, 25);

  _hLeptonChargeAfter  = DeclareTH1F("LeptonChargeAfter", "Sign of top quark charge after cut", 7, -2, 5);
  _hLeptonFlavorAfter  = DeclareTH1F("LeptonFlavorAfter", "Flavor code (kf) of lepton after" ,50, -25, 25);

  // Get configuration
  EventContainer *EventContainerObj = GetEventContainer();
  TEnv *config = EventContainerObj -> GetConfig();

  // Use configuration to set min and max number of jets to cut on
  _leptonCharge = config -> GetValue("Cut.Lepton.Charge", 1);
  _leptonFlavor = config -> GetValue("Cut.Lepton.Flavor", -1);

} // BookHistogram

/******************************************************************************
 * Bool_t CutCharge::Apply()                                                  *
 *                                                                            *
 * Apply cuts and fill histograms                                             *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTrue if successful                                                *
 ******************************************************************************/
Bool_t CutCharge::Apply()
{

  // Lepton Variables on which to cut
  Int_t leptonCharge = -5;
  Int_t leptonFlavor = -5;
  Int_t leptonNumber =  0;
  Bool_t numberPass  = kFALSE;
  Bool_t chargePass  = kFALSE;
  Bool_t flavorPass  = kFALSE;

  // Get Event Container
  EventContainer *EventContainerObj = GetEventContainer();
  
  // Get Variables from lepton vector - make sure a lepton exists
  // Currently cutting only on ELECTRONS
  leptonNumber = EventContainerObj -> electrons.size();
  if(leptonNumber > 0){
    numberPass = kTRUE;
    leptonCharge = static_cast<Int_t>(EventContainerObj -> electrons[0].Charge());
    cout << "Lepton Charge " << leptonCharge << endl;
  } //if

  // Fill the histogram before the cuts
  _hLeptonChargeBefore -> Fill(leptonCharge);
  _hLeptonFlavorBefore -> Fill(leptonFlavor);

  // Cut on Charge
  // Keep event with charge less than -1 or greater than 1
  if( (_leptonCharge < -1) || (_leptonCharge > 1) || (_leptonCharge == leptonCharge) ) chargePass = kTRUE;

  // No cut on flavor yet
  flavorPass = kTRUE;
    
  // Fill the histograms after the cut
  if( numberPass && chargePass && flavorPass ) {
    _hLeptonChargeAfter -> Fill(leptonCharge);
    _hLeptonFlavorAfter -> Fill(leptonFlavor);
  } //if


  return ( numberPass && chargePass && flavorPass );
  
} //Apply()












