/******************************************************************************
 * CutNJets.hpp                                                               *
 *                                                                            *
 * Cuts on number of jets in an event                                         *
 *                                                                            *
 * Derived from HistoCut which is in turn derived from BaseCut                *
 *                                                                            *
 * Public Member Functions of AnalysisMain class                              *
 *    CutNJets()                        -- Parameterized Constructor          *
 *    ~CutNJets()                       -- Destructor                         *
 *    BookHistogram()                   -- Book histograms                    *
 *    Apply()                           -- Apply cuts and fill histograms     *
 *    GetCutName()                      -- Returns "CutNJets"                 *
 *                                                                            *
 * Private Data Members of AnalysisMain class                                 *
 *    myTH1F* _hNJetsBefore              -- Hist of Num of jets before cuts   *
 *    myTH1F* _hNJetsAfter               -- Hist of Num of jets before cuts   *
 *                                                                            *
 * History                                                                    *
 *       7 Nov 2006 - Created by R. Schwienhorst for ATLAS                    *
 *      10 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *      20 Nov 2006 - RS: add interface to cutFlowTable.                      *
 *****************************************************************************/

#include "single_top_analysis/CutElectron.hpp"
#include <iostream>
#include <sstream>

using namespace std;


/******************************************************************************
 * CutElectron::CutElectron(EventContainer *EventContainerObj)                *
 *                                                                            *
 * Parameterized Constructor                                                  *
 *                                                                            *
 * Input:  Event Object class                                                 *
 * Output: None                                                               *
 ******************************************************************************/
CutElectron::CutElectron(EventContainer *EventContainerObj)
{
  SetEventContainer(EventContainerObj);
} // CutElectron


/******************************************************************************
 * CutElectron::~CutElectron()                                                *
 *                                                                            *
 * Destructor                                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
CutElectron::~CutElectron()
{
  
}//~CutElectron

/******************************************************************************
 * void CutElectron::BookHistogram()                                          *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void CutElectron::BookHistogram(){
  
  // Histogram of events before cut on number of jets
  // Number of Electrons
  _hElectronNumberBefore =  DeclareTH1F("ElectronNumberBefore","N_{e} before cut",10, 0, 10.0);
  _hElectronNumberBefore -> SetXAxisTitle("N_{e}");
  _hElectronNumberBefore -> SetYAxisTitle("Events");

  // Electron Pt
  _hElectronPtBefore =  DeclareTH1F("ElectronPtBefore","Electron p_{T} before cut",105, -10, 200);
  _hElectronPtBefore -> SetXAxisTitle("Electron p_{T}");
  _hElectronPtBefore -> SetYAxisTitle("Events");

  // Histogram of events after cut on number of electrons
  // Number of Electrons
  _hElectronNumberAfter =  DeclareTH1F("ElectronNumberAfter","N_{e} after cut",10, 0, 10);
  _hElectronNumberAfter -> SetXAxisTitle("N_{e}");
  _hElectronNumberAfter -> SetYAxisTitle("Events");

  // Electron Pt
  _hElectronPtAfter =  DeclareTH1F("ElectronPtAfter","Electron p_{T} after cut",100, 0.0, 200.0);
  _hElectronPtAfter -> SetXAxisTitle("Electron p_{T}");
  _hElectronPtAfter -> SetYAxisTitle("Events");
  
  // Get configuration
  EventContainer *EventContainerObj = GetEventContainer();
  TEnv *config = EventContainerObj -> GetConfig();

  // Use configuration to set min and max number of jets to cut on
  _ElectronNumberMin = config -> GetValue("Cut.Electron.Number.Min", 0);
  _ElectronNumberMax = config -> GetValue("Cut.Electron.Number.Max", -1);
  _ElectronPtMin     = config -> GetValue("Cut.Electron.Pt.Min", 0);
  _ElectronPtMax     = config -> GetValue("Cut.Electron.Pt.Max", -1);
  
  //
  // also add these cuts to the cut flow table
  ostringstream titleStr;
  titleStr << "Electron (" << _ElectronPtMin<<" <= PT <= "<<_ElectronPtMax<<"): ";
  titleStr <<_ElectronNumberMin<<" <= N <= "<<_ElectronNumberMax;
  TString tTitle(titleStr.str().c_str());

  GetCutFlowTable()->AddCutToFlow("Electron.All",tTitle);
  
}//BookHistograms()

/******************************************************************************
 * Bool_t CutElectron::Apply()                                                *
 *                                                                            *
 * Apply cuts and fill histograms                                             *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTrue if successful                                                *
 ******************************************************************************/
Bool_t CutElectron::Apply()
{

  // Electron Variables on which to cut
  Int_t ElectronNumber  = 0;         // Number of electrons
  Bool_t eventPass      = kFALSE;    // Event passes ALL electron cuts

  // Get Event Container
  EventContainer *EventContainerObj = GetEventContainer();

  // Get Variables from electron vector in the EventContainer Object

  for(int ie=0;ie<EventContainerObj -> electrons.size();ie++) {
    if( (EventContainerObj -> electrons[ie].Pt() >= _ElectronPtMin) // min PT check
	&& (_ElectronPtMax<0. || (EventContainerObj -> electrons[ie].Pt() <= _ElectronPtMax) // max PT, if defined
	    ) 
	) {
      ElectronNumber++;
    } // end of if electron pt
    // Fill the histogram before the cut
    _hElectronPtBefore     -> Fill(EventContainerObj -> electrons[ie].Pt());
  } // end of for loop

  if( ElectronNumber >= _ElectronNumberMin
      && ElectronNumber <= _ElectronNumberMax ){
    eventPass = kTRUE;
  } //if

  // Fill the histogram before the cut
  _hElectronNumberBefore -> Fill(ElectronNumber);

  // Fill the histogram after the cut
  if(eventPass){
    _hElectronNumberAfter -> Fill(ElectronNumber);
    GetCutFlowTable()->PassCut("Electron.All");

    // in order to fill the electron pT after the cut,
    // we need to consider only those electrons that pass,
    // thus, we need to make the same if condition as above.
    for(int ie=0;ie<EventContainerObj -> electrons.size();ie++) {
      if( (EventContainerObj -> electrons[ie].Pt() >= _ElectronPtMin) // min PT check
	  && (_ElectronPtMax<0. || (EventContainerObj -> electrons[ie].Pt() <= _ElectronPtMax) // max PT, if defined
	      ) 
	  ) {
	_hElectronPtAfter     -> Fill(EventContainerObj -> electrons[ie].Pt());
      } // end of if
    } // end of for loop

  } 
  else {
    GetCutFlowTable()->FailCut("Electron.All");
  }
  
  return eventPass;  
  
} //Apply












