/******************************************************************************
 * CutFlowTable.cpp                                                           *
 *                                                                            *
 * Keep track of events passing/failing cuts.                                 *
 *                                                                            *
 *                                                                            *
 * Public Member Functions of this class                                      *
 *    CutFlowTable()      -- Parameterized Constructor                        *
 *    ~CutFlowTable()     -- Destructor                                       *
 *    Clear()            -- Reset internal counters                           *
 *    PrintCutFlow()     -- Print out cut statistics                          *
 *    AddCutToFlow()     -- Add a new cut to the table                        *
 *    PassCut()          -- Tell this class about a cut that passed           *
 *    FailCut()          -- Tell this class about a cut that failed           *
 *    FindCutName()      -- Returns true if this cut already exists           *
 *    SetEventContainer()-- Set the internal eventContainer pointer.          *
 *                                                                            *
 * Private Data Members of this class                                         *
 * vector<pair<TString,CutFlowLine> >: list of cuts                           *
 * EventContainer*: pointer to event container                                *
 *                                                                            *
 * History                                                                    *
 *      21 Nov 2006 - Created by R. Schwienhorst for ATLAS                    *
 *****************************************************************************/
#include "single_top_analysis/CutFlowTable.hpp"
#include "EventContainer.hpp"

#include <iostream>
#include <iomanip>

using namespace std;

//
// default constructor
//  First one is most likely used only by root during readback.
//
CutFlowTable::CutFlowTable(void)
  : TNamed ("NONE",""), _EventObj(NULL)
{
  _cutLines.clear();
}

CutFlowTable::CutFlowTable(const TString& name, const TString& title)
  : TNamed (name.Data(), title.Data()), _EventObj(NULL)
{
  _cutLines.clear();
}

//
// Print
//
void CutFlowTable::PrintCutFlow(Option_t *option) const
{
  cout << "<CutFlowTable> " << GetName() << " result:"<<endl;
  cout << "---------------------------------------------------------"<<endl;
  cout << "    Cut Name     |      exposed      |        pass       |"<<endl; 
  cout << "                 |  count  |  yield  |  count  |  yield  |"<<endl; 
  for(int iCut=0; iCut<_cutLines.size(); iCut++) {
    _cutLines[iCut].second.Print(option);
  }
  cout << "---------------------------------------------------------"<<endl;
   
}

//
// AddCut
//
//  Add a new cut to the list of cuts
//
void CutFlowTable::AddCutToFlow(const TString& name, const TString& title)
{

  // first check if this cut is present
  if(FindCutName(name)>=0) {
    cout<<"<CutFlowTable::AddCutToFlow> "<<"cut "<<name<<" is already in the list!"<<endl;    
  }
  else {
    CutFlowLine newline(name,title);
    _cutLines.push_back(make_pair(name,newline));
  }
}

//
// FailCut
//
//  This cut failed, and we are not given a weight, use the global weight.
//
void CutFlowTable::FailCut(const TString& name)
{
  Double_t wgt=1.;
  if(_EventObj) wgt = _EventObj->GetGlobalEventWeight();
  FailCut(name,wgt);
}

//
// FailCut
//
//  This cut failed, and we are given a weight. 
//
void CutFlowTable::FailCut(const TString& name, const Double_t& weight)
{

  // first check if this cut is present
  if(FindCutName(name)<0) {
    cout<<"<CutFlowTable::FailCut> "<<"cut "<<name<<" not found! Adding it to the list"<<endl;    
    CutFlowLine newline(name);
    _cutLines.push_back(make_pair(name,newline));
  }
  _cutLines[FindCutName(name)].second.FailCut(weight);
}

//
// PassCut
//
//  This cut passed, and we are not given a weight, use the global weight.
//
void CutFlowTable::PassCut(const TString& name)
{
  Double_t wgt=1.;
  if(_EventObj) wgt = _EventObj->GetGlobalEventWeight();
  PassCut(name,wgt);
}

//
// PassCut
//
//  This cut passed, and we are given a weight. Inc the guy!
//
void CutFlowTable::PassCut(const TString& name, const Double_t& weight)
{

  // first check if this cut is present
  if(FindCutName(name)<0) {
    cout<<"<CutFlowTable::PassCut> "<<"cut "<<name<<" not found! Adding it to the list"<<endl;    
    CutFlowLine newline(name);
    pair<TString,CutFlowLine> newEntry(name,newline);
    _cutLines.push_back(newEntry);
  }
  _cutLines[FindCutName(name)].second.PassCut(weight);
}

//
// FindCutName
//
// Find the index of the cut with this name in the vector.
// Returns -1 if this cut name is not found.
//
Int_t CutFlowTable::FindCutName(const TString& name) const
{
  Int_t res=-1;
  for(int i=0;i<_cutLines.size();i++) {
    if(_cutLines[i].first==name) {
      return i;
    }
  }
  return -1;
}
