/******************************************************************************
 * CutNJets.hpp                                                               *
 *                                                                            *
 * Cuts on number of jets in an event                                         *
 *                                                                            *
 * Derived from HistoCut which is in turn derived from BaseCut                *
 *                                                                            *
 * Public Member Functions of CutNJet class                                   *
 *    CutNJets()                        -- Parameterized Constructor          *
 *    ~CutNJets()                       -- Destructor                         *
 *    BookHistogram()                   -- Book histograms                    *
 *    Apply()                           -- Apply cuts and fill histograms     *
 *    GetCutName()                      -- Returns "CutNJets"                 *
 *                                                                            *
 * Private Data Members of CutNJet class                                      *
 *    myTH1F* _hNJetsBefore              -- Hist of Num of jets before cuts   *
 *    myTH1F* _hNJetsAfter               -- Hist of Num of jets before cuts   *
 *    Int_t _nJetsMin;                   -- Minimum number of jets to require *
 *    Int_t _nJetsMax;                   -- Maximum number of jets allowed    *
 *                                                                            *
 * History                                                                    *
 *       7 Nov 2006 - Created by R. Schwienhorst for ATLAS                    *
 *      10 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *      17 Nov 2006 - RS: add interface to cutFlowTable.                      *
 *****************************************************************************/

#include "single_top_analysis/CutNJets.hpp"
#include <iostream>
#include <sstream>

using namespace std;


/******************************************************************************
 * CutNJets::CutNJets(EventContainer *EventContainerObj)                      *
 *                                                                            *
 * Parameterized Constructor                                                  *
 *                                                                            *
 * Input:  Event Object class                                                 *
 * Output: None                                                               *
 ******************************************************************************/
CutNJets::CutNJets(EventContainer *EventContainerObj)
{
  SetEventContainer(EventContainerObj);
} // CutNJets


/******************************************************************************
 * CutNJets::~CutNJets()                                                      *
 *                                                                            *
 * Destructor                                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
CutNJets::~CutNJets()
{
  
}//~CutNJets

/******************************************************************************
 * void CutNJets::BookHistogram()                                             *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void CutNJets::BookHistogram(){
  
  // Histogram of events before cut on number of jets
  _hNJetsBefore =  DeclareTH1F("NJetsBefore","Number of Jets before the cut",10, 0.0, 10.0);
  _hNJetsBefore -> SetXAxisTitle("N_{Jets}");
  _hNJetsBefore -> SetYAxisTitle("Events");

  // Histogram of events after cut on number of jets
  _hNJetsAfter =  DeclareTH1F("NJetsAfter","Number of Jets after the cut",10, 0.0, 10.0);
  _hNJetsAfter -> SetXAxisTitle("N_{Jets}");
  _hNJetsAfter -> SetYAxisTitle("Events");
  
  // Get configuration
  EventContainer *EventContainerObj = GetEventContainer();
  TEnv *config = EventContainerObj -> GetConfig();

  // Use configuration to set min and max number of jets to cut on
  _nJetsMin = config -> GetValue("Cut.Jet.Njets.Min", 0);
  _nJetsMax = config -> GetValue("Cut.Jet.Njets.Max", -1);

  //
  // also add these two cuts to the cut flow table
  ostringstream titleStr;
  titleStr <<"Jets: "<< _nJetsMin<<" <= N";
  TString tTitle(titleStr.str().c_str());
  GetCutFlowTable()->AddCutToFlow("NJets.Min",tTitle);
  titleStr.str("");
  titleStr <<"Jets: N <= "<<_nJetsMax;
  tTitle=titleStr.str().c_str();
  GetCutFlowTable()->AddCutToFlow("NJets.Max",tTitle);
  // also add the overal pass rate
  titleStr.str("");
  titleStr <<"Jets: "<< _nJetsMin<<" <= N <= "<<_nJetsMax;
  tTitle=titleStr.str().c_str();
  GetCutFlowTable()->AddCutToFlow("NJets.All",tTitle);

  
}//BookHistograms()

/******************************************************************************
 * Bool_t CutNJets::Apply()                                                   *
 *                                                                            *
 * Apply cuts and fill histograms                                             *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTrue if successful                                                *
 ******************************************************************************/
Bool_t CutNJets::Apply()
{
  //cout<<"Begin of CutNJets::Apply()"<<endl;

  // Get Event Tree
  EventContainer *evObj = GetEventContainer();

  Int_t njets=evObj->jets.size();
  // Fill the histogram before the cut
  _hNJetsBefore -> Fill(njets);

  // Flag to signify if event passes cut
  Bool_t eventPass = kTRUE;

  // Cut on Min number of jets
  if(njets < _nJetsMin) {
    GetCutFlowTable()->FailCut("NJets.Min");
    eventPass = kFALSE;
  }
  else {
    GetCutFlowTable()->PassCut("NJets.Min");
  }

  // Cut on Max number of jets
  // Negative cut value for Max means that there is no Max cut
  if(eventPass) {
    if( (_nJetsMax >= 0) && (njets > _nJetsMax) ) {
      GetCutFlowTable()->FailCut("NJets.Max");
      eventPass = kFALSE;
    }
    else {
      GetCutFlowTable()->PassCut("NJets.Max");
    }
  }

  // Fill the histogram after the cut
  if(eventPass) {
    _hNJetsAfter -> Fill(njets);
    GetCutFlowTable()->PassCut("NJets.All");
  }
  else {
    GetCutFlowTable()->FailCut("NJets.All");
  }

  //cout<<"End of CutNJets::Apply()"<<endl;
  return eventPass;  
  
} //Apply












