/******************************************************************************
 * EventContainer.cpp                                                         *
 *                                                                            *
 * Read in variables from EventTree and put them into an Object               *
 *                                                                            *
 * Public Members of EventContainer    class                                  *
 *    EventContainer()                  -- Default Constructor                *
 *    ~EventContainer()                 -- Destructor                         *
 *    GetSourceName()                   -- Get name of source being run on    *
 *    GetSourceNumber()                 -- Get number of source being run on  *
 *    GetTargetTopMass()                -- Get Top Quark Mass                 *
 *    SetSourceName()                   -- Set name of source being run on    *
 *    SetSourceNumber()                 -- Set number of source being run on  *
 *    SetTargetTopMass()                -- Set Top Quark Mass                 *
 *    Initialize()                      -- Initialize class                   *
 *    MakeTopQuarks()                   -- Reconstruct Top in several ways    *
 *    GetNextEvent()                    -- Set up particles for next event    *
 *    SetEventCount()                   -- Set counter for specific event     *
 *    GetCollectionTree()               -- Get the collection tree            *
 *    GetEventTree()                    -- Get the event tree                 *
 *    GetTruthTree()                    -- Get the truth tree                 *
 *    GetDebugLevel()                   -- Get debug level                    *
 *    SetDebugLevel()                   -- Set debug level                    *
 *    GetEventWeight()                  -- Get Event Weight                   *
 *    GetOutputEventWeight()            -- Get Output EventWeight             *
 *    GetTreeEventWeight()              -- Get Tree Event Weight              *
 *    GetGlobalEventWeight()            -- Get Global EventWeight             *
 *    SetGlobalEventWeight()            -- Set global event weight            *
 *    SetOutputEventWeight()            -- Set output event weight            *
 *    GetConfig()                       -- Get Configuration                  *
 *                                                                            *
 *    Int_t ReadEvent()                 -- Create TLorentz Vectors            *
 *    Bool_t CreateFinalState()         -- Create final state particles       *
 *    Int_t runNumber                   -- Run number from the root tree      *
 *    Int_t eventNumber                 -- Event number from the root tree    *
 *    vector<Electron> electrons        -- Electron Vector                    *
 *    vector<Muon>     muons            -- Muon Vector                        *
 *    vector<Tau>      taus             -- Tau Vector                         *
 *    vector<Jet>      jets             -- Jet Vector                         *
 *    vector<MCParticle> MCparticles    -- MC Particle Vector                 *
 *                                                                            *
 * Protected Members of EventContainer class                                  *
 *    Int_t debugLevel                  -- Debug level                        *
 *    TEnv _config                      -- Used to read configuration         *
 *                                                                            *
 * Private Members of EventContainer class                                    *
 *    Double_t _targetTopMass           -- Target top mass                    *
 *    CollectionTree *_collectionTree   -- Collection tree                    *
 *    EventTree *_eventTree             -- Event tree                         *
 *    TruthTree *_thruthTree            -- Collection tree                    *
 *    Int_t _eventCount                 -- Location in event chain            *
 *    TString _sourceName               -- Name of source being read          *
 *    TString _sourceNumber             -- Number of source being read        *
 *    Double_t _globalEventWeight       -- Global Event Weight                *
 *    Double_t _treeEventWeight         -- Tree Event Weight                  *
 *    Double_t _outputEventWeight       -- Output Event Weight                *
 *                                                                            *
 *    Apply()                           -- Apply cuts and fill histograms     *
 *    GetCutName()                      -- Returns "CutNJets"                 *
 *                                                                            *
 * Private Data Members of CutNJets                                           *
 *    myTH1F* _hNJetsBefore              -- Hist of Num of jets before cuts   *
 *    myTH1F* _hNJetsAfter               -- Hist of Num of jets before cuts   *
 *    Int_t _nJetsMin;                   -- Minimum number of jets to require *
 *    Int_t _nJetsMax;                   -- Maximum number of jets allowed    *
 *                                                                            *
 * History                                                                    *
 *      19 Oct 2006 - Created by R. Schwienhorst for ATLAS                    *
 *      14 Nov 2006 - Modified by R. Schwienhorst - added tau, muon classes   *
 *      16 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#include "single_top_analysis/EventContainer.hpp"
#include <iostream>
#include <string>
#include <fstream>

#include "single_top_analysis/utils.hpp"

using namespace std;

// Integrate classes into the Root system
ClassImp(EventContainer)


//-----------------------------------------------------------------------------------------
//
// Get the next event.
// It reads in the next event from the current chain and 
// Creates final state particles.
// This method returns -1 when there are no more events.
// It returns the total event count if successful.
Int_t EventContainer::GetNextEvent()
{
  //cout<<"EventContainer::GetNextEvent, count "<<_eventCount<<endl;
  Int_t countOffSet=0;
  // check that the chains are defined:
  if (_collectionTree->fChain==0) return -1;


  // now Set things up
  Int_t bytesread = _collectionTree->fChain->GetEntry(_eventCount);
  
  if(bytesread==0) return -1;

  // Create 4-vectors
  ReadEvent();



  // Create final state objects like W bosons and top quarks
  //finalStatePass=CreateFinalState();
  
  // and finally increment the internal event counter
  _eventCount++;
  //cout<<"returning from GetNextEvent"<<endl;
  return(countOffSet+_eventCount-1);
} // end of GetNextEvent
  


/******************************************************************************
 * EventContainer::EventContainer()                                           *
 *                                                                            *
 * Default Constructor                                                        *
 * Initializes many private data members                                      *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
EventContainer::EventContainer():_collectionTree(NULL), _eventTree(NULL), _truthTree(NULL),
				 _eventCount(0), _targetTopMass(175.),
				 _debugLevel(0),
				 _sourceName("NONE"),
				 _globalEventWeight(1.), _treeEventWeight(1.), _outputEventWeight(1.),
				 _config("Configuration")
{
 
} //EventContainer()


/******************************************************************************
 * EventContainer::~EventContainer()                                          *
 *                                                                            *
 * Destructor                                                                 *
 * Sets pointers to NULL                                                      *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
EventContainer::~EventContainer()
{
  // Collection Tree
  if(_collectionTree) delete _collectionTree;
  _collectionTree = NULL;

  // Event Tree
  if(_eventTree) delete _eventTree;
  _eventTree = NULL;

  // Truth Tree
  if(_truthTree) delete _truthTree;
  _truthTree = NULL;
} //~EventContainer()

/******************************************************************************
 * void EventContainer::Initialize()                                          *
 *                                                                            *
 * Initialize class                                                           *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void EventContainer::Initialize(CollectionTree* collectionTree,EventTree* eventTree, TruthTree* truthTree)
{

  // Initialize the analysis classes
  _collectionTree = collectionTree;
  _eventTree = eventTree;
  _truthTree = truthTree;

  // now Set up the internal variables:
  _eventCount = 0;
  electrons.clear();
  muons.clear();
  taus.clear();
  jets.clear();
  taggedJets.clear();
  unTaggedJets.clear();
  MCparticles.clear();

  return;
} //Initialize()


/******************************************************************************
 * Int_t EventContainer::ReadEvent()                                          *
 *                                                                            *
 * Fill objects with event info                                               *
 *                                                                            *
 * Input:  None                                                               *
 * Output: Return 0                                                           *
 ******************************************************************************/
Int_t EventContainer::ReadEvent()
{
  // Set the event weight if there is any
  _treeEventWeight = _eventTree -> eventWeight;

  runNumber   = _eventTree -> runNumber;
  eventNumber = _eventTree -> eventNumber;

  // Reset all of the particle vectors
  electrons.clear();
  muons.clear();
  taus.clear();
  jets.clear();
  taggedJets.clear();
  unTaggedJets.clear();
  MCparticles.clear();


  /////////////////////////////////////////////////////
  // Fill objects
  /////////////////////////////////////////////////////

  // Electrons
  Electron newElectron;
  for(Int_t io = 0; io < _eventTree -> El_N; io++) {
    newElectron.Clear();
    newElectron.Fill(_eventTree, io);
    electrons.push_back(newElectron);
  } //for

  // Muons
  Muon newMuon;
  for(Int_t io = 0;io < _eventTree -> Mu_N; io++) {
    newMuon.Clear();
    newMuon.Fill(_eventTree, io);
    muons.push_back(newMuon);
  } //for

  // Taus
  Tau newTau;
  for(Int_t io = 0; io < _eventTree -> Tau_N; io++) {
    newTau.Clear();
    newTau.Fill(_eventTree, io);
    taus.push_back(newTau);
  } //for

  // Jets, also check tagging info
  Jet newJet;
  for(Int_t io = 0;io < _eventTree -> PJet_N; io++) {
    newJet.Clear();
    newJet.Fill(_eventTree, io);
    jets.push_back(newJet);
    //
    // check if it is tagged or not
    bool isTagged=false;
    //
    // need code here!
    //
    // now fill the corresponding vector
    if(isTagged) {
      taggedJets.push_back(newJet);
    }
    else {
      unTaggedJets.push_back(newJet);
    }


  } //loop over jets

  // All MC particles
  MCParticle newMCp;
  for(Int_t io = 0;io < _truthTree -> Truth_N; io++) {
    newMCp.Clear();
    newMCp.Fill(_truthTree, io);
    MCparticles.push_back(newMCp);
  } //for

  return 0;
} //ReadEvent

/******************************************************************************
 * Bool_t EventContainer::CreateFinalState()                                  *
 *                                                                            *
 * Create final state particles                                               *
 * Require at least 2 jets and 1 isolated lepton.  Make cuts if necessary.    *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTRUE if requirements fulfilled                                    *
 ******************************************************************************/
Bool_t EventContainer::CreateFinalState()
{
//   //cout<<"begin of particles::Create final state"<<endl;

  //finalStatePass=kTRUE;
  
//   //
//   // for the neutrino we pretend that we don't know where it is.
//   // All we can do is reconstruct it.
//   Double_t nu_px = -tll.Px();
//   Double_t nu_py = -tll.Py();
//   for(Int_t i=0;i<njets;i++) {
//     nu_px += -finalJets[i].Px();
//     nu_py += -finalJets[i].Py();
//   }
//   // last argument to neutrino_pz is a large negative number
//   // to indicate that we don't want to use a scale factor.
//   Double_t scf=-99999.;
//   //cout<<"calling pzneutrino"<<endl;
//   Double_t nu_pz=Neutrino_Pz(nu_px,nu_py,
// 			  tll.Px(),tll.Py(),tll.Pz(),tll.E(),scf,tlnu.Pz());
//   //

//   // in the D0 analysis we use the scale factor, but here we don't
//   // because we know neutrino px and py very well.
//   if(fabs(nu_pz)>tlnu.Pz()) {
//     //cout<<" making neutrino, initial PZ is "<<tlnu.Pz()<<", final Pz "<<nu_pz<<endl;
//   }
//   Double_t nu_e=sqrt(nu_px*nu_px + nu_py*nu_py + nu_pz*nu_pz);
//   finalNeutrino.SetPxPyPzE(nu_px,nu_py,nu_pz,nu_e);
//   finalNeutrino.SetPdgId(-12);
  

//   // now reconstruct the W and the top quark
//   //cout<<"making W"<<endl;
//   finalW = finalNeutrino;
//   finalW += finalLepton;
//   //cout<<"final W p "<<finalW.P()<<", E "<<finalW.E()<<", M "<<finalW.M()<<endl;

//   //cout<<"Assigning jets to b and bbar quarks"<<endl;
//   // and assign the final jets to the b and bbar quark
//   bJetPtr=0;
//   bbarJetPtr=0;
//   gluonJetPtr=0;
//   lqJetPtr=0;
//   for(Int_t i=0;i<njets;i++) {
//     if(finalJets[i].hasPdgId(5)) {
//       // this is the b quark jet
//       bJetPtr=&(finalJets[i]);
//       //cout<<"Jet "<<i<<" is the b."<<endl;
//     }
//     if(finalJets[i].hasPdgId(-5)) {
//       // this is the bbar quark jet
//       bbarJetPtr=&(finalJets[i]);
//       //cout<<"Jet "<<i<<" is the bbar."<<endl;
//     }
//     if(finalJets[i].hasPdgId(21)) {
//       // this is the gluon jet
//       gluonJetPtr=&(finalJets[i]);
//       //cout<<"Jet "<<i<<" is the gluon."<<endl;
//     }
//     if(finalJets[i].hasPdgId(1)) {
//       // this is the light quark jet
//       lqJetPtr=&(finalJets[i]);
//       //cout<<"Jet "<<i<<" is the light quark."<<endl;
//     }
//   } // end of loop over jets
//   //
//   // we are not yet re-creating the jets from the top quark here,
//   // instead we are waiting for the user to do so explicitly.
//   // simply reSet all of the pointers at this point.
//   finalTop_leadingJet.SetPxPyPzE(0.,0.,0.,0.);
//   finalTop_bestJet.SetPxPyPzE(0.,0.,0.,0.);
//   finalTop_bestJet12.SetPxPyPzE(0.,0.,0.,0.);
//   finalTop_bgJet.SetPxPyPzE(0.,0.,0.,0.);
//   bestJet12Ptr=0;
//   nonBestJet12Ptr=0;
//   bestJetPtr=0;
//   bgJet.SetPxPyPzE(0.,0.,0.,0.);
//   bgJetbJetPtr=0;
//   nonBgJetPtr=0;
//   bgJetGluonPtr=0;
//   taggedJetPtr=0;
//   untaggedPtJetPtr=0;
//   untaggedEtaJetPtr=0;
//   forwardJetPtr=0;
  
  //cout<<"end of particles::Create final state"<<endl;
  //return finalStatePass;
  return kTRUE;
} //CreateFinalState() 

/******************************************************************************
 * void EventContainer::MakeTopQuarks()                                       *
 *                                                                            *
 * Make top quarks from W and several jets                                    *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void EventContainer::MakeTopQuarks()
{
  //
  // make top quarks from the W and several possible jets. 
//   // here we can Create several top quarks
//   // start by making one from the w and the leading jet
//   finalTop_leadingJet = finalW;
//   finalTop_leadingJet += finalJets[0];
  
  
//   //cout<<"finding best jet"<<endl;
//   // now use the best jet (the one that reconstructs to a
//   // top mass closest to the target top mass (default=175GeV)
//   TLorentzVector tempJets[4];
//   for(Int_t iJet=0;iJet<njets;iJet++) tempJets[iJet]=finalJets[iJet];

//   Int_t ibestjet = findBestJet(njets,tempJets,finalW,_targetTopMass);
//   bestJetPtr=&(finalJets[ibestjet]);
//   finalTop_bestJet = finalW;
//   finalTop_bestJet += *bestJetPtr;
//   // for bestjet12 only consider leading two jets
//   Int_t itemp=njets>2? 2: njets;
//   ibestjet = findBestJet(itemp,tempJets,finalW,_targetTopMass);
//   bestJet12Ptr=&(finalJets[ibestjet]);
//   finalTop_bestJet12 = finalW;
//   finalTop_bestJet12 += *(bestJet12Ptr);
  
//   // the non-best jet
//   if(njets>=2) {
//     nonBestJet12Ptr=&(finalJets[1-ibestjet]);
//   } else {
//     nonBestJet12Ptr=0;
//   }

//   //////////////////////////////////////////////////////////////////////
//   //
//   //cout<<"add in a possible gluon"<<endl;
//   // add in the gluon if it is close to this jet
//   // and then determine the best jet
//   if(njets<=2) {
//     // nothing to be done, bestJet12 is the same as this
//     finalTop_bgJet=finalTop_bestJet12;
//     bgJet=*bestJet12Ptr;
//     bgJetbJetPtr=bestJet12Ptr;
//     nonBgJetPtr=nonBestJet12Ptr;
//   }
//   else {
//     // there are three jets in the event, pair each of the leading two
//     // with the gluon and then find the best jet
//     for(Int_t i=0;i<2;i++) {
//       tempJets[i]=finalJets[i];
//       tempJets[i+2] = finalJets[i];
//       tempJets[i+2] += finalJets[2];
//     } // end of loop over jets 1,2
//     //cout<<"finding best jet out of 4"<<endl;
//     ibestjet=findBestJet(4,tempJets,finalW,_targetTopMass);
//     finalTop_bgJet=tempJets[ibestjet];
//     finalTop_bgJet += finalW;
//     bgJet=tempJets[ibestjet];
//     // figure out if we included the gluon or not.
//     if(ibestjet>=2) {
//       bgJetbJetPtr=&(finalJets[ibestjet-2]);
//       nonBgJetPtr=&(finalJets[1-(ibestjet-2)]);
//       bgJetGluonPtr=&(finalJets[2]);
//     }
//     else {
//       bgJetbJetPtr=&(finalJets[ibestjet]);
//       nonBgJetPtr=&(finalJets[1-(ibestjet)]);
//       bgJetGluonPtr=0;
//     }
//   } // end of else: try to include the gluon

//   // final top from tagged jet and W
//   if(taggedJetPtr!=0) {
//     finalTop_taggedJet=*taggedJetPtr;
//     finalTop_taggedJet += finalW;
    
//     // and the W using a top mass constraint:
//     // the neutrino from the top mass constraint
//     // also use a constraint on the top mass rather than the smaller of the two
//     // solutions.
//     //cout<<"True Top mass is "<<tlt.M()<<" e= "<<tlt.E()<<", px= "<<tlt.Px()<<", py= "<<tlt.Py()<<", pz= "<<tlt.Pz()<<endl;
//     Double_t nu_pz2=Neutrino_Pz_Top(finalNeutrino.Px(),finalNeutrino.Py(),
// 				  finalLepton.Px(),finalLepton.Py(),finalLepton.Pz(),finalLepton.E(),
// 				  taggedJetPtr->Px(),taggedJetPtr->Py(),taggedJetPtr->Pz(),taggedJetPtr->E(),
// 				  tlnu.Pz());
    
    
//     ////
//     Double_t nu_e=sqrt(pow(finalNeutrino.Px(),2) + pow(finalNeutrino.Py(),2) + nu_pz2*nu_pz2);
//     finalNeutrinoTopC.SetPxPyPzE(finalNeutrino.Px(),finalNeutrino.Py(),nu_pz2,nu_e);
//     finalNeutrinoTopC.SetPdgId(-12);
    
//     // the final lepton from top constraint on neutrino
//     finalWTopC = finalNeutrinoTopC;
//     finalWTopC += finalLepton;

//     finalTop_taggedJetTopC =*taggedJetPtr;
//     finalTop_taggedJetTopC += finalWTopC;
//   }
//   else {
//     finalTop_taggedJet.SetPxPyPzE(0.,0.,0.,0.);
//     finalTop_taggedJetTopC.SetPxPyPzE(0.,0.,0.,0.);
//   }


//   //Particle tempt=finalW;
//   //tempt += bJet;
//   //cout<<"top mass "<<tempt.M()<<", "<<finalTop_bestJet12.M();
//   //cout<<" final jets: "<<(finalW+finalJets[0]).M()<<", "<<(finalW+finalJets[1]).M()<<endl;
//   //cout<<"best jet pt, phi is "<<bestJet12.Pt()<<", "<<bestJet12.Phi()<<endl;
//   //cout<<"   b jet pt, phi is "<<bJet.Pt()<<", "<<bJet.Phi()<<endl;


  return;
} //MakeTopQuark

