/******************************************************************************
 * HistogrammingElectron.cpp                                                  *
 *                                                                            *
 * Books and fills histograms                                                 *
 * Used for events passing cuts applied in other classes                      *
 *                                                                            *
 * Derived from HistoCut which is in turn derived from BaseCut                *
 *                                                                            *
 * Public Member Functions of AnalysisMain class                              *
 *    HistogrammingElectron()              -- Parameterized Constructor       *
 *    ~HistogrammingElectron()             -- Destructor                      *
 *    BookHistogram()                   -- Book histograms                    *
 *    Apply()                           -- Fill histograms only (No Cuts)     *
 *    GetCutName()                      -- Returns "HistogrammingElectron"    *
 *                                                                            *
 * Private Data Members of this class                                         *
 * Lots of histograms of electron properties                                  *
 *                                                                            *
 * History                                                                    *
 *      14 Nov 2006 - Created by R. Schwienhorst for ATLAS                    *
 *****************************************************************************/

#include "single_top_analysis/HistogrammingElectron.hpp"
#include<iostream>

using namespace std;

/******************************************************************************
 * HistogrammingElectron::HistogrammingElectron(EventContainer *obj)          *
 *                                                                            *
 * Parameterized Constructor                                                  *
 *                                                                            *
 * Input:  Event container                                                    *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingElectron::HistogrammingElectron(EventContainer *obj)
{
  SetEventContainer(obj);
} //HistogrammingElectron()

/******************************************************************************
 * HistogrammingElectron::~HistogrammingElectron()                                  *
 *                                                                            *
 * Destructor                                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingElectron::~HistogrammingElectron()
{
  
} //HistogrammingElectron

/******************************************************************************
 * void HistogrammingElectron::BookHistogram()                                          *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void HistogrammingElectron::BookHistogram(){
  
  // Histogram of Number of Objects
  _hNObj = DeclareTH1F("EleN","Number of Electrons",10,0.,10.);
  _hNObj->SetXAxisTitle("N");
  _hNObj->SetYAxisTitle("Events");
  // Electron 1 Pt
  _hPtObj1 = DeclareTH1F("Ele1Pt","Electron 1 P_{T}",200,0.,400.);
  _hPtObj1->SetXAxisTitle("P_{T} (electron 1) [GeV]");
  _hPtObj1->SetYAxisTitle("Events");
  // Electron 1 Eta
  _hEtaObj1 = DeclareTH1F("Ele1Eta","Electron 1 Pseudorapidity",100,-5.,5.);
  _hEtaObj1->SetXAxisTitle("#eta (electron 1) ");
  _hEtaObj1->SetYAxisTitle("Events");
  // Electron 1 Phi
  _hPhiObj1 = DeclareTH1F("Ele1Phi","Electron 1 phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhiObj1->SetXAxisTitle("#phi (electron 1)");
  _hPhiObj1->SetYAxisTitle("Events");
  // Electron 2 Pt
  _hPtObj2 = DeclareTH1F("Ele2Pt","Electron 2 P_{T}",200,0.,400.);
  _hPtObj2->SetXAxisTitle("P_{T} (electron 2) [GeV]");
  _hPtObj2->SetYAxisTitle("Events");
  // Electron 2 Eta
  _hEtaObj2 = DeclareTH1F("Ele2Eta","Electron 2 Pseudorapidity",100,-5.,5.);
  _hEtaObj2->SetXAxisTitle("#eta (electron 2) ");
  _hEtaObj2->SetYAxisTitle("Events");
  // Electron 2 Phi
  _hPhiObj2 = DeclareTH1F("Ele2Phi","Electron 2 phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhiObj2->SetXAxisTitle("#phi (electron 2)");
  _hPhiObj2->SetYAxisTitle("Events");

  // any Electron Pt
  _hPt = DeclareTH1F("ElePt","any Electron P_{T}",200,0.,400.);
  _hPt->SetXAxisTitle("P_{T} (electron) [GeV]");
  _hPt->SetYAxisTitle("Events");
  // any Electron Eta
  _hEta = DeclareTH1F("EleEta","any Electron Pseudorapidity",100,-5.,5.);
  _hEta->SetXAxisTitle("#eta (electron) ");
  _hEta->SetYAxisTitle("Events");
  // any Electron Phi
  _hPhi = DeclareTH1F("ElePhi","Any electron phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhi->SetXAxisTitle("#phi (electron)");
  _hPhi->SetYAxisTitle("Events");
  // any Electron Charge
  _hCharge = DeclareTH1F("EleCharge","Any electron charge",3,-1.5,1.5);
  _hCharge->SetXAxisTitle("charge (electron)");
  _hCharge->SetYAxisTitle("Events");
  // any Electron E over P
  _hEoverP = DeclareTH1F("EleEoverP","Any electron Energy over Momentum",100,0.,10.);
  _hEoverP->SetXAxisTitle("E/P (electron)");
  _hEoverP->SetYAxisTitle("Events");
  
  //cout<<"end of HistogrammingElectron::BookHistogram"<<endl;

} //BookHistogram

/******************************************************************************
 * Bool_t HistogrammingElectron::Apply()                                         *
 *                                                                            *
 * Fill histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTrue if successful                                                *
 ******************************************************************************/
Bool_t HistogrammingElectron::Apply()
{
  //cout<<"Begin of HistogrammingElectron::Apply()"<<endl;
  
  // Get Event Tree
  EventContainer *evc = GetEventContainer();
  
  // Fill Histograms
  _hNObj -> Fill(evc->electrons.size());

  if(evc->electrons.size()>0) {
    _hPtObj1  -> Fill(evc->electrons[0].Pt());
    _hEtaObj1 -> Fill(evc->electrons[0].Eta());
    _hPhiObj1 -> Fill(evc->electrons[0].Phi());
  }
  if(evc->electrons.size()>1) {
    _hPtObj2  -> Fill(evc->electrons[1].Pt());
    _hEtaObj2 -> Fill(evc->electrons[1].Eta());
    _hPhiObj2 -> Fill(evc->electrons[1].Phi());
  }

  // now loop over all electrons and histogram their properties
  for(int io=0;io<evc->electrons.size();io++) {
    _hPt  -> Fill(evc->electrons[io].Pt());
    _hEta -> Fill(evc->electrons[io].Eta());
    _hPhi -> Fill(evc->electrons[io].Phi());
    _hCharge->Fill(evc->electrons[io].GetCharge());
    _hEoverP->Fill(evc->electrons[io].GetEoverP());
  }    

  //cout<<"End of HistogrammingElectron::Apply()"<<endl;
  return kTRUE;  
  
} //Apply












