/******************************************************************************
 * HistogrammingTopology.cpp                                                  *
 *                                                                            *
 * Books and fills histograms                                                 *
 * Used for events passing cuts applied in other classes                      *
 *                                                                            *
 * Derived from HistoCut which is in turn derived from BaseCut                *
 *                                                                            *
 * Public Member Functions of AnalysisMain class                              *
 *    HistogrammingTopology()              -- Parameterized Constructor       *
 *    ~HistogrammingTopology()             -- Destructor                      *
 *    BookHistogram()                   -- Book histograms                    *
 *    Apply()                           -- Fill histograms only (No Cuts)     *
 *    GetCutName()                      -- Returns "HistogrammingTopology"    *
 *                                                                            *
 * Private Data Members of this class                                         *
 * Lots of histograms of topological variables                                *
 *                                                                            *
 * History                                                                    *
 *      21 Nov 2006 - Created by R. Schwienhorst for ATLAS                    *
 *****************************************************************************/

#include "single_top_analysis/HistogrammingTopology.hpp"
#include<iostream>

using namespace std;

/******************************************************************************
 * HistogrammingTopology::HistogrammingTopology(EventContainer *obj)          *
 *                                                                            *
 * Parameterized Constructor                                                  *
 *                                                                            *
 * Input:  Event container                                                    *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingTopology::HistogrammingTopology(EventContainer *obj) : _skimFile(NULL),_skimTree(NULL)
{
  SetEventContainer(obj);
} //HistogrammingTopology()

/******************************************************************************
 * HistogrammingTopology::~HistogrammingTopology()                            *
 *                                                                            *
 * Destructor                                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingTopology::~HistogrammingTopology()
{
} //~HistogrammingTopology

/******************************************************************************
 * void HistogrammingTopology::BookHistogram()                                *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void HistogrammingTopology::BookHistogram()
{
  // Histogram of Number of jets
  _hNJet = DeclareTH1F("JetN","Number of Jets",10,0.,10.);
  _hNJet->SetXAxisTitle("N (jets)");
  _hNJet->SetYAxisTitle("Events");
  
  // Histogram of Number of jets
  _hNUnTaggedJet = DeclareTH1F("UnTagJetN","Number of untagged Jets",10,0.,10.);
  _hNUnTaggedJet->SetXAxisTitle("N (jets)");
  _hNUnTaggedJet->SetYAxisTitle("Events");
  
  // Histogram of Number of jets
  _hNTaggedJet = DeclareTH1F("TagJetN","Number of tagged Jets",10,0.,10.);
  _hNTaggedJet->SetXAxisTitle("N (tagged jets)");
  _hNTaggedJet->SetYAxisTitle("Events");
  
  // Histogram of Number of Electrons
  _hNEle = DeclareTH1F("EleN","Number of Electrons",10,0.,10.);
  _hNEle->SetXAxisTitle("N (electrons)");
  _hNEle->SetYAxisTitle("Events");
  
  // Histogram of Number of muons
  _hNMu = DeclareTH1F("MuN","Number of Muons",10,0.,10.);
  _hNMu->SetXAxisTitle("N (muons)");
  _hNMu->SetYAxisTitle("Events");
  
  // Histogram of Number of jets
  _hNTau = DeclareTH1F("TauN","Number of Taus",10,0.,10.);
  _hNTau->SetXAxisTitle("N (taus)");
  _hNTau->SetYAxisTitle("Events");

  //
  // now check the options: should we make a skim file?
  
  // Get configuration
  EventContainer *EventContainerObj = GetEventContainer();
  TEnv *config = EventContainerObj -> GetConfig();

  // Use configuration to set name of output skim file
  if(config->Defined("Topology.SkimFile")) {
    _skimFileName= config -> GetValue("Topology.SkimFile", "NONE"); 
    cout<<"HistogrammingTopology: writing out topological variables tree "<<_skimFileName<<endl;
  
    _skimFile = new TFile(_skimFileName,"RECREATE");
    _skimTree = new TTree("Topology", "Topological Variables");

    //
    // now add branches to our new tree
    // global event info:
    _skimTree->Branch("EventWeight", &_eventWeight, "_eventWeight/D");
    _skimTree->Branch("RunNumber", &_runNumber, "_runNumber/I");
    _skimTree->Branch("EventNumber", &_eventNumber, "_eventNumber/I");
    _skimTree->Branch("SourceNumber",&_sourceNumber,"_sourceNumber/I");

    // object counters
    _skimTree->Branch("NJet", &_nJet, "_nJet/I");
    _skimTree->Branch("NUnTagJet", &_nUnTaggedJet, "_nUnTaggedJet/I");
    _skimTree->Branch("NTagJet", &_nTaggedJet, "_nTaggedJet/I");
    _skimTree->Branch("NElectron", &_nEle, "_nEle/I");
    _skimTree->Branch("NMuon", &_nMu, "_nMu/I");
    _skimTree->Branch("NTau", &_nTau, "_nTau/I");


  } // end of: if skim file name is defined

  
  //cout<<"end of HistogrammingTopology::BookHistogram"<<endl;

} //BookHistogram

/******************************************************************************
 * Bool_t HistogrammingTopology::Apply()                                         *
 *                                                                            *
 * Fill histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTrue if successful                                                *
 ******************************************************************************/
Bool_t HistogrammingTopology::Apply()
{
  //cout<<"Begin of HistogrammingTopology::Apply()"<<endl;
  
  // Get Event Tree
  EventContainer *evc = GetEventContainer();
  
  // Fill Histograms

  _hNJet -> Fill(evc->jets.size());
  _hNTaggedJet -> Fill(evc->taggedJets.size());
  _hNUnTaggedJet -> Fill(evc->unTaggedJets.size());
  _hNEle -> Fill(evc->electrons.size());
  _hNMu -> Fill(evc->muons.size());
  _hNTau -> Fill(evc->taus.size());


  //
  // If we are writing out the skim tree, fill it here
  if(_skimTree!=NULL) {

    _eventWeight  = evc->GetEventWeight();
    _runNumber    = evc->runNumber;
    _eventNumber  = evc->eventNumber;
    _sourceNumber = evc->GetSourceNumber();

    _nJet         = evc->jets.size();
    _nUnTaggedJet = evc->unTaggedJets.size();
    _nTaggedJet   = evc->taggedJets.size();
    _nMu          = evc->muons.size();
    _nTau         = evc->taus.size();
    _nEle         = evc->electrons.size();

    
    _skimTree->Fill();
  } // end of: if filling skim tree



  //cout<<"End of HistogrammingTopology::Apply()"<<endl;
  return kTRUE;  
  
} //Apply

/******************************************************************************
 * void HistogrammingTopology::SaveHistogram()                                *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void HistogrammingTopology::SaveHistogram()
{
  // if we were writing out the skim file, then close it here.
  if(_skimTree!=NULL) {
    _skimFile->Write();    
    _skimFile->Close();
    delete(_skimFile);
    _skimFile=NULL;
    cout<<"<HistogrammingTopology> Written topo tree to file "<<_skimFileName<<endl;
  }
  

}









